@@ -3,7 +3,6 @@
 #include <stdbool.h>
 
 #include <osmocom/core/logging.h>
-#include <osmocom/core/thread.h>
 
 extern const struct log_info log_info;
 
@@ -19,10 +18,6 @@ enum {
 	DCTR,
 };
 
-#define CLOGC(category, level, fmt, args...) do { \
-	LOGP(category, level, "[tid=%ld] " fmt, (long int) osmo_gettid(), ##args);  \
-} while(0)
-
 #define CLOGCHAN(chan, category, level, fmt, args...) do { \
-	LOGP(category, level, "[tid=%ld][chan=%zu] " fmt, (long int) osmo_gettid(), chan, ##args);  \
+	LOGP(category, level, "[chan=%zu] " fmt, chan, ##args);  \
 } while(0)
