@@ -80,7 +80,7 @@ struct TransceiverState {
 
   /* Received noise energy levels */
   float mNoiseLev;
-  noiseVector mNoises;
+  avgVector mNoises;
 
   /* Shadowed downlink attenuation */
   int mPower;
@@ -148,6 +148,7 @@ public:
   } ChannelCombination;
 
 private:
+  size_t mChans;
 struct ctrl_msg {
   char data[101];
   ctrl_msg() {};
@@ -161,9 +162,9 @@ struct ctrl_sock_state {
   }
   ~ctrl_sock_state() {
       if(conn_bfd.fd >= 0) {
+          osmo_fd_unregister(&conn_bfd);
           close(conn_bfd.fd);
           conn_bfd.fd = -1;
-          osmo_fd_unregister(&conn_bfd);
       }
   }
 };
@@ -218,7 +219,7 @@ struct ctrl_sock_state {
   /** drive handling of control messages from GSM core */
   int ctrl_sock_handle_rx(int chan);
 
-  size_t mChans;
+
   bool mOn;	                           ///< flag to indicate that transceiver is powered on
   bool mForceClockInterface;           ///< flag to indicate whether IND CLOCK shall be sent unconditionally after transceiver is started
   bool mHandover[8][8];                ///< expect handover to the timeslot/subslot
