@@ -6,6 +6,8 @@
  *
  * Author: Tom Tsou <tom@tsou.cc>
  *
+ * SPDX-License-Identifier: AGPL-3.0+
+ *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -57,9 +59,9 @@ static size_t resamp_inchunk = 0;
 static size_t resamp_outrate = 0;
 static size_t resamp_outchunk = 0;
 
-RadioInterfaceResamp::RadioInterfaceResamp(RadioDevice *wRadio,
+RadioInterfaceResamp::RadioInterfaceResamp(RadioDevice *wDevice,
 					   size_t tx_sps, size_t rx_sps)
-	: RadioInterface(wRadio, tx_sps, rx_sps, 1),
+	: RadioInterface(wDevice, tx_sps, rx_sps, 1),
 	  outerSendBuffer(NULL), outerRecvBuffer(NULL)
 {
 }
@@ -96,15 +98,6 @@ bool RadioInterfaceResamp::init(int type)
 {
 	float cutoff = 1.0f;
 
-	close();
-
-	sendBuffer.resize(1);
-	recvBuffer.resize(1);
-	convertSendBuffer.resize(1);
-	convertRecvBuffer.resize(1);
-	mReceiveFIFO.resize(1);
-	powerScaling.resize(1);
-
 	switch (type) {
 	case RadioDevice::RESAMP_64M:
 		resamp_inrate = RESAMP_64M_INRATE;
@@ -169,7 +162,7 @@ int RadioInterfaceResamp::pullBuffer()
 		return -1;
 
 	/* Outer buffer access size is fixed */
-	num_recv = mRadio->readSamples(convertRecvBuffer,
+	num_recv = mDevice->readSamples(convertRecvBuffer,
 				       resamp_outchunk,
 				       &overrun,
 				       readTimestamp,
@@ -182,7 +175,7 @@ int RadioInterfaceResamp::pullBuffer()
 	convert_short_float((float *) outerRecvBuffer->begin(),
 			    convertRecvBuffer[0], 2 * resamp_outchunk);
 
-	underrun |= local_underrun;
+	osmo_trx_sync_or_and_fetch(&underrun, local_underrun);
 	readTimestamp += (TIMESTAMP) resamp_outchunk;
 
 	/* Write to the end of the inner receive buffer */
@@ -202,6 +195,7 @@ int RadioInterfaceResamp::pullBuffer()
 /* Send a timestamped chunk to the device */
 bool RadioInterfaceResamp::pushBuffer()
 {
+	bool local_underrun;
 	int rc;
 	size_t numSent;
 
@@ -221,14 +215,15 @@ bool RadioInterfaceResamp::pushBuffer()
 			    (float *) outerSendBuffer->begin(),
 			    powerScaling[0], 2 * resamp_outchunk);
 
-	numSent = mRadio->writeSamples(convertSendBuffer,
+	numSent = mDevice->writeSamples(convertSendBuffer,
 				       resamp_outchunk,
-				       &underrun,
+				       &local_underrun,
 				       writeTimestamp);
 	if (numSent != resamp_outchunk) {
 		LOG(ALERT) << "Transmit error " << numSent;
 	}
 
+	osmo_trx_sync_or_and_fetch(&underrun, local_underrun);
 	writeTimestamp += resamp_outchunk;
 
 	return true;
