@@ -12,6 +12,8 @@ enum {
 	DNB,
 	DMM,
 	DCC,
+	DGCC,
+	DBCC,
 	DSS,
 	DSMS,
 	DMNCC,
@@ -26,6 +28,13 @@ enum {
 	DPRIM,
 	DLUA,
 	DGAPK,
+	DCSD,
+	DTUN,
+	DRLCMAC,
+	DLLC,
+	DSNDCP,
+	DGMM,
+	DSM
 };
 
 extern const struct log_info log_info;
