@@ -22,6 +22,7 @@
 #include <stdlib.h>
 
 #include <osmocom/core/talloc.h>
+#include <osmocom/gsm/protocol/gsm_04_08.h>
 
 #include <osmocom/bb/common/logging.h>
 #include <osmocom/bb/common/osmocom_data.h>
@@ -34,7 +35,15 @@
 static uint32_t new_callref = 1;
 static LLIST_HEAD(call_list);
 
-static int dtmf_statemachine(struct gsm_call *call, struct gsm_mncc *mncc);
+static const char * const gsm_call_type_str[] = {
+	[GSM_CALL_T_UNKNOWN]	= "unknown",
+	[GSM_CALL_T_VOICE]	= "voice",
+	[GSM_CALL_T_DATA]	= "data",
+	[GSM_CALL_T_DATA_FAX]	= "fax",
+};
+
+static int dtmf_statemachine(struct gsm_call *call,
+			     const struct gsm_mncc *mncc);
 static void timeout_dtmf(void *arg);
 
 /*
@@ -80,72 +89,72 @@ struct gsm_call *get_call_ref(uint32_t callref)
 	return NULL;
 }
 
-static int8_t mncc_get_bearer(struct gsm_settings *set, uint8_t speech_ver)
+static int8_t mncc_get_bearer(const struct gsm_settings *set, uint8_t speech_ver)
 {
 	switch (speech_ver) {
-	case 4:
+	case GSM48_BCAP_SV_AMR_F:
 		if (set->full_v3)
 			LOGP(DMNCC, LOGL_INFO, " net suggests full rate v3\n");
 		else {
 			LOGP(DMNCC, LOGL_INFO, " full rate v3 not supported\n");
-			speech_ver = -1;
+			return -1;
 		}
 		break;
-	case 2:
+	case GSM48_BCAP_SV_EFR:
 		if (set->full_v2)
 			LOGP(DMNCC, LOGL_INFO, " net suggests full rate v2\n");
 		else {
 			LOGP(DMNCC, LOGL_INFO, " full rate v2 not supported\n");
-			speech_ver = -1;
+			return -1;
 		}
 		break;
-	case 0: /* mandatory */
+	case GSM48_BCAP_SV_FR: /* mandatory */
 		if (set->full_v1)
 			LOGP(DMNCC, LOGL_INFO, " net suggests full rate v1\n");
 		else {
 			LOGP(DMNCC, LOGL_INFO, " full rate v1 not supported\n");
-			speech_ver = -1;
+			return -1;
 		}
 		break;
-	case 5:
+	case GSM48_BCAP_SV_AMR_H:
 		if (set->half_v3)
 			LOGP(DMNCC, LOGL_INFO, " net suggests half rate v3\n");
 		else {
 			LOGP(DMNCC, LOGL_INFO, " half rate v3 not supported\n");
-			speech_ver = -1;
+			return -1;
 		}
 		break;
-	case 1:
+	case GSM48_BCAP_SV_HR:
 		if (set->half_v1)
 			LOGP(DMNCC, LOGL_INFO, " net suggests half rate v1\n");
 		else {
 			LOGP(DMNCC, LOGL_INFO, " half rate v1 not supported\n");
-			speech_ver = -1;
+			return -1;
 		}
 		break;
 	default:
 		LOGP(DMNCC, LOGL_INFO, " net suggests unknown speech version "
 			"%d\n", speech_ver);
-		speech_ver = -1;
+		return -1;
 	}
 
 	return speech_ver;
 }
 
-static void mncc_set_bearer(struct osmocom_ms *ms, int8_t speech_ver,
-	struct gsm_mncc *mncc)
+static void mncc_set_bcap_speech(struct gsm_mncc *mncc,
+				 const struct gsm_settings *set,
+				 int speech_ver)
 {
-	struct gsm_settings *set = &ms->settings;
 	int i = 0;
 
 	mncc->fields |= MNCC_F_BEARER_CAP;
-	mncc->bearer_cap.coding = 0;
+	mncc->bearer_cap.coding = GSM48_BCAP_CODING_GSM_STD;
 	if (set->ch_cap == GSM_CAP_SDCCH_TCHF_TCHH
 	 && (set->half_v1 || set->half_v3)) {
-		mncc->bearer_cap.radio = 3;
+		mncc->bearer_cap.radio = GSM48_BCAP_RRQ_DUAL_FR;
 		LOGP(DMNCC, LOGL_INFO, " support TCH/H also\n");
 	} else {
-		mncc->bearer_cap.radio = 1;
+		mncc->bearer_cap.radio = GSM48_BCAP_RRQ_FR_ONLY;
 		LOGP(DMNCC, LOGL_INFO, " support TCH/F only\n");
 	}
 	mncc->bearer_cap.speech_ctm = 0;
@@ -153,41 +162,379 @@ static void mncc_set_bearer(struct osmocom_ms *ms, int8_t speech_ver,
 	if (speech_ver < 0) {
 		/* if half rate is supported and preferred */
 		if (set->half_v3 && set->half && set->half_prefer) {
-			mncc->bearer_cap.speech_ver[i++] = 5;
+			mncc->bearer_cap.speech_ver[i++] = GSM48_BCAP_SV_AMR_H;
 			LOGP(DMNCC, LOGL_INFO, " support half rate v3\n");
 		}
 		if (set->half_v1 && set->half && set->half_prefer) {
-			mncc->bearer_cap.speech_ver[i++] = 1;
+			mncc->bearer_cap.speech_ver[i++] = GSM48_BCAP_SV_HR;
 			LOGP(DMNCC, LOGL_INFO, " support half rate v1\n");
 		}
 		/* if full rate is supported */
 		if (set->full_v3) {
-			mncc->bearer_cap.speech_ver[i++] = 4;
+			mncc->bearer_cap.speech_ver[i++] = GSM48_BCAP_SV_AMR_F;
 			LOGP(DMNCC, LOGL_INFO, " support full rate v3\n");
 		}
 		if (set->full_v2) {
-			mncc->bearer_cap.speech_ver[i++] = 2;
+			mncc->bearer_cap.speech_ver[i++] = GSM48_BCAP_SV_EFR;
 			LOGP(DMNCC, LOGL_INFO, " support full rate v2\n");
 		}
 		if (set->full_v1) { /* mandatory, so it's always true */
-			mncc->bearer_cap.speech_ver[i++] = 0;
+			mncc->bearer_cap.speech_ver[i++] = GSM48_BCAP_SV_FR;
 			LOGP(DMNCC, LOGL_INFO, " support full rate v1\n");
 		}
 		/* if half rate is supported and not preferred */
 		if (set->half_v3 && set->half && !set->half_prefer) {
-			mncc->bearer_cap.speech_ver[i++] = 5;
+			mncc->bearer_cap.speech_ver[i++] = GSM48_BCAP_SV_AMR_H;
 			LOGP(DMNCC, LOGL_INFO, " support half rate v3\n");
 		}
 		if (set->half_v1 && set->half && !set->half_prefer) {
-			mncc->bearer_cap.speech_ver[i++] = 1;
+			mncc->bearer_cap.speech_ver[i++] = GSM48_BCAP_SV_HR;
 			LOGP(DMNCC, LOGL_INFO, " support half rate v1\n");
 		}
 	/* if specific speech_ver is given (it must be supported) */
 	} else
 		mncc->bearer_cap.speech_ver[i++] = speech_ver;
 	mncc->bearer_cap.speech_ver[i] = -1; /* end of list */
-	mncc->bearer_cap.transfer = 0;
-	mncc->bearer_cap.mode = 0;
+	mncc->bearer_cap.transfer = GSM48_BCAP_ITCAP_SPEECH;
+	mncc->bearer_cap.mode = GSM48_BCAP_TMOD_CIRCUIT;
+}
+
+static const struct bcap_data_set {
+	enum gsm48_bcap_ra		ra;
+	enum gsm48_bcap_interm_rate	ir;
+	enum gsm48_bcap_user_rate	ur;
+	enum gsm48_bcap_modem_type	mt;
+} bcap_data_set_map[] = {
+	[DATA_CALL_TR_V21_300] = {
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_300,
+		.mt = GSM48_BCAP_MT_V21,
+	},
+	[DATA_CALL_TR_V22_1200] = {
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_1200,
+		.mt = GSM48_BCAP_MT_V22,
+	},
+	[DATA_CALL_TR_V23_1200_75] = {
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_1200_75,
+		.mt = GSM48_BCAP_MT_V23,
+	},
+	[DATA_CALL_TR_V22bis_2400] = {
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_2400,
+		.mt = GSM48_BCAP_MT_V22bis,
+	},
+	[DATA_CALL_TR_V26ter_2400] = {
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_2400,
+		.mt = GSM48_BCAP_MT_V26ter,
+	},
+	[DATA_CALL_TR_V32_4800] = {
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_4800,
+		.mt = GSM48_BCAP_MT_V32,
+	},
+	[DATA_CALL_TR_V32_9600] = {
+		.ir = GSM48_BCAP_IR_16k,
+		.ur = GSM48_BCAP_UR_9600,
+		.mt = GSM48_BCAP_MT_V32,
+	},
+#if 0
+	[DATA_CALL_TR_V34_9600] = {
+		.ir = GSM48_BCAP_IR_16k,
+		.ur = GSM48_BCAP_UR_9600,
+		.mt = GSM48_BCAP_MT_V32,
+		/* (Octet 6c) Modem type: According to ITU-T Rec. V.32
+		 * (Octet 6d) Other modem type: According to ITU-T Rec. V.34 (2)
+		 * TODO: gsm48_encode_bearer_cap() does not support octet 6d */
+	},
+#endif
+	[DATA_CALL_TR_V110_300] = {
+		.ra = GSM48_BCAP_RA_V110_X30,
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_300,
+	},
+	[DATA_CALL_TR_V110_1200] = {
+		.ra = GSM48_BCAP_RA_V110_X30,
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_1200,
+	},
+	[DATA_CALL_TR_V110_2400] = {
+		.ra = GSM48_BCAP_RA_V110_X30,
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_2400,
+	},
+	[DATA_CALL_TR_V110_4800] = {
+		.ra = GSM48_BCAP_RA_V110_X30,
+		.ir = GSM48_BCAP_IR_8k,
+		.ur = GSM48_BCAP_UR_4800,
+	},
+	[DATA_CALL_TR_V110_9600] = {
+		.ra = GSM48_BCAP_RA_V110_X30,
+		.ir = GSM48_BCAP_IR_16k,
+		.ur = GSM48_BCAP_UR_9600,
+	},
+};
+
+static void mncc_set_bcap_data(struct gsm_mncc *mncc,
+			       const struct gsm_settings *set,
+			       enum gsm_call_type call_type)
+{
+	const struct data_call_params *cp = &set->call_params.data;
+	struct gsm_mncc_bearer_cap *bcap = &mncc->bearer_cap;
+	const struct bcap_data_set *ds;
+
+	OSMO_ASSERT(cp->type_rate < ARRAY_SIZE(bcap_data_set_map));
+	ds = &bcap_data_set_map[cp->type_rate];
+	if (ds->ir == 0 && ds->ur == 0) {
+		LOGP(DMNCC, LOGL_ERROR, "AT+CBST=%d is not supported\n", cp->type_rate);
+		return;
+	}
+
+	mncc->fields |= MNCC_F_BEARER_CAP;
+
+	*bcap = (struct gsm_mncc_bearer_cap) {
+		/* .transfer is set below */
+		.mode = GSM48_BCAP_TMOD_CIRCUIT,
+		.coding = GSM48_BCAP_CODING_GSM_STD,
+		/* .radio is set below */
+		.data = {
+			.sig_access = GSM48_BCAP_SA_I440_I450,
+			.rate_adaption = ds->ra,
+			.interm_rate = ds->ir,
+			.user_rate = ds->ur,
+			.modem_type = ds->mt,
+			.transp = cp->transp,
+
+			/* async call params */
+			.async = cp->is_async,
+			.nr_data_bits = cp->nr_data_bits,
+			.nr_stop_bits = cp->nr_stop_bits,
+			.parity = cp->parity,
+		},
+	};
+
+	/* Radio channel requirement (octet 3) */
+	if (set->ch_cap == GSM_CAP_SDCCH_TCHF_TCHH) {
+		if (set->half_prefer)
+			bcap->radio = GSM48_BCAP_RRQ_DUAL_HR;
+		else
+			bcap->radio = GSM48_BCAP_RRQ_DUAL_FR;
+		LOGP(DMNCC, LOGL_INFO, " support TCH/H also\n");
+	} else {
+		bcap->radio = GSM48_BCAP_RRQ_FR_ONLY;
+		LOGP(DMNCC, LOGL_INFO, " support TCH/F only\n");
+	}
+
+	/* Information transfer capability (octet 3) */
+	switch (call_type) {
+	case GSM_CALL_T_DATA:
+		if (ds->mt == GSM48_BCAP_MT_NONE)
+			bcap->transfer = GSM_MNCC_BCAP_UNR_DIG;
+		else /* analog modem */
+			bcap->transfer = GSM_MNCC_BCAP_AUDIO;
+		break;
+	case GSM_CALL_T_DATA_FAX:
+		bcap->transfer = GSM_MNCC_BCAP_FAX_G3;
+		break;
+	case GSM_CALL_T_VOICE:
+	default: /* shall not happen */
+		OSMO_ASSERT(0);
+	}
+
+	/* FAX calls are special (see 3GPP TS 24.008, Annex D.3) */
+	if (call_type == GSM_CALL_T_DATA_FAX) {
+		bcap->data.rate_adaption = GSM48_BCAP_RA_NONE;
+		bcap->data.async = 0; /* shall be sync */
+		bcap->data.transp = GSM48_BCAP_TR_TRANSP;
+		bcap->data.modem_type = GSM48_BCAP_MT_NONE;
+	}
+}
+
+/* Check the given Bearer Capability, select first supported speech codec version.
+ * The choice between half-rate and full-rate is made based on current settings.
+ * Return a selected codec or -1 if no speech codec was selected. */
+static int mncc_handle_bcap_speech(const struct gsm_mncc_bearer_cap *bcap,
+				   const struct gsm_settings *set)
+{
+	int speech_ver_half = -1;
+	int speech_ver = -1;
+
+	for (unsigned int i = 0; bcap->speech_ver[i] >= 0; i++) {
+		int temp = mncc_get_bearer(set, bcap->speech_ver[i]);
+		switch (temp) {
+		case GSM48_BCAP_SV_AMR_H:
+		case GSM48_BCAP_SV_HR:
+			/* only the first half rate */
+			if (speech_ver_half < 0)
+				speech_ver_half = temp;
+			break;
+		default:
+			if (temp < 0)
+				continue;
+			/* only the first full rate */
+			if (speech_ver < 0)
+				speech_ver = temp;
+			break;
+		}
+	}
+
+	/* half and full given */
+	if (speech_ver_half >= 0 && speech_ver >= 0) {
+		if (set->half_prefer) {
+			LOGP(DMNCC, LOGL_INFO, " both supported"
+				" codec rates are given, using "
+				"preferred half rate\n");
+			speech_ver = speech_ver_half;
+		} else {
+			LOGP(DMNCC, LOGL_INFO, " both supported"
+				" codec rates are given, using "
+				"preferred full rate\n");
+		}
+	} else if (speech_ver_half < 0 && speech_ver < 0) {
+		LOGP(DMNCC, LOGL_INFO, " no supported codec "
+			"rate is given\n");
+	/* only half rate is given, use it */
+	} else if (speech_ver_half >= 0) {
+		LOGP(DMNCC, LOGL_INFO, " only supported half "
+			"rate codec is given, using it\n");
+		speech_ver = speech_ver_half;
+	/* only full rate is given, use it */
+	} else {
+		LOGP(DMNCC, LOGL_INFO, " only supported full "
+			"rate codec is given, using it\n");
+	}
+
+	return speech_ver;
+}
+
+/* Check the given Bearer Capability for a data call (CSD).
+ * Return 0 if the bearer is accepted, otherwise return -1. */
+static int mncc_handle_bcap_data(const struct gsm_mncc_bearer_cap *bcap,
+				 const struct gsm_settings *set)
+{
+	switch (bcap->transfer) {
+	case GSM48_BCAP_ITCAP_UNR_DIG_INF:
+		if (bcap->data.rate_adaption != GSM48_BCAP_RA_V110_X30) {
+			LOGP(DMNCC, LOGL_ERROR,
+			     "%s(): Rate adaption (octet 5) 0x%02x is not supported\n",
+			     __func__, bcap->data.rate_adaption);
+			return -ENOTSUP;
+		}
+		break;
+	case GSM48_BCAP_ITCAP_3k1_AUDIO:
+	case GSM48_BCAP_ITCAP_FAX_G3:
+		if (bcap->data.rate_adaption != GSM48_BCAP_RA_NONE) {
+			LOGP(DMNCC, LOGL_ERROR,
+			     "%s(): Rate adaption (octet 5) 0x%02x was expected to be NONE\n",
+			     __func__, bcap->data.rate_adaption);
+			return -ENOTSUP;
+		}
+		break;
+	default:
+		break;
+	}
+
+	if (bcap->data.sig_access != GSM48_BCAP_SA_I440_I450) {
+		LOGP(DMNCC, LOGL_ERROR,
+		     "%s(): Signalling access protocol (octet 5) 0x%02x is not supported\n",
+		     __func__, bcap->data.sig_access);
+		return -ENOTSUP;
+	}
+
+#define BCAP_RATE(interm_rate, user_rate) \
+	((interm_rate << 8) | (user_rate << 0))
+
+	switch (BCAP_RATE(bcap->data.interm_rate, bcap->data.user_rate)) {
+	case BCAP_RATE(GSM48_BCAP_IR_8k, GSM48_BCAP_UR_300):
+	case BCAP_RATE(GSM48_BCAP_IR_8k, GSM48_BCAP_UR_1200):
+	case BCAP_RATE(GSM48_BCAP_IR_8k, GSM48_BCAP_UR_2400):
+		if (bcap->data.transp != GSM48_BCAP_TR_TRANSP) {
+			LOGP(DMNCC, LOGL_ERROR,
+			     "%s(): wrong user-rate 0x%02x for a non-transparent call\n",
+			     __func__, bcap->data.user_rate);
+			return -EINVAL;
+		}
+		/* fall-through */
+	case BCAP_RATE(GSM48_BCAP_IR_8k, GSM48_BCAP_UR_4800):
+	case BCAP_RATE(GSM48_BCAP_IR_16k, GSM48_BCAP_UR_9600):
+		if (bcap->data.transp != GSM48_BCAP_TR_TRANSP) {
+			LOGP(DMNCC, LOGL_ERROR,
+			     "%s(): only transparent calls are supported so far\n",
+			     __func__);
+			return -ENOTSUP;
+		}
+		break;
+	default:
+		LOGP(DMNCC, LOGL_ERROR,
+		     "%s(): IR 0x%02x / UR 0x%02x combination is not supported\n",
+		     __func__, bcap->data.interm_rate, bcap->data.user_rate);
+		return -ENOTSUP;
+	}
+
+#undef BCAP_RATE
+
+	return 0;
+}
+
+static int mncc_handle_bcap(struct gsm_mncc *mncc_out,		/* CC Call Confirmed */
+			    const struct gsm_mncc *mncc_in,	/* CC Setup */
+			    const struct gsm_settings *set)
+{
+	const struct gsm_mncc_bearer_cap *bcap = &mncc_in->bearer_cap;
+
+	/* 3GPP TS 24.008, section 9.3.2.2 defines several cases in which the
+	 * Bearer Capability 1 IE is to be included, provided that at least
+	 * one of these conditions is met. */
+
+	/* if the Bearer Capability 1 IE is not present */
+	if (~mncc_in->fields & MNCC_F_BEARER_CAP) {
+		/* ... include our own Bearer Capability, assuming a speech call */
+		mncc_set_bcap_speech(mncc_out, set, -1);
+		return 0;
+	}
+
+	if (bcap->mode != GSM48_BCAP_TMOD_CIRCUIT) {
+		LOGP(DMNCC, LOGL_ERROR,
+		     "%s(): Transfer mode 0x%02x is not supported\n",
+		     __func__, bcap->mode);
+		return -ENOTSUP;
+	}
+	if (bcap->coding != GSM48_BCAP_CODING_GSM_STD) {
+		LOGP(DMNCC, LOGL_ERROR,
+		     "%s(): Coding standard 0x%02x is not supported\n",
+		     __func__, bcap->coding);
+		return -ENOTSUP;
+	}
+
+	switch (bcap->transfer) {
+	case GSM48_BCAP_ITCAP_SPEECH:
+	{
+		int speech_ver = mncc_handle_bcap_speech(bcap, set);
+		/* include bearer cap, if not given in setup (see above)
+		 * or if multiple codecs are given
+		 * or if not only full rate
+		 * or if given codec is unimplemented
+		 */
+		if (speech_ver < 0)
+			mncc_set_bcap_speech(mncc_out, set, -1);
+		else if (bcap->speech_ver[1] >= 0 || speech_ver != 0)
+			mncc_set_bcap_speech(mncc_out, set, speech_ver);
+		break;
+	}
+	case GSM48_BCAP_ITCAP_UNR_DIG_INF:
+	case GSM48_BCAP_ITCAP_3k1_AUDIO:
+	case GSM48_BCAP_ITCAP_FAX_G3:
+		return mncc_handle_bcap_data(bcap, set);
+	default:
+		LOGP(DMNCC, LOGL_ERROR,
+		     "%s(): Information transfer capability 0x%02x is not supported\n",
+		     __func__, bcap->transfer);
+		return -ENOTSUP;
+	}
+
+	return 0;
 }
 
 /*
@@ -257,12 +604,11 @@ int mncc_recv_external(struct osmocom_ms *ms, int msg_type, void *arg)
 
 int mncc_recv_internal(struct osmocom_ms *ms, int msg_type, void *arg)
 {
-	struct gsm_settings *set = &ms->settings;
-	struct gsm_mncc *data = arg;
+	const struct gsm_settings *set = &ms->settings;
+	const struct gsm_mncc *data = arg;
 	struct gsm_call *call = get_call_ref(data->callref);
 	struct gsm_mncc mncc;
 	uint8_t cause;
-	int8_t	speech_ver = -1, speech_ver_half = -1, temp;
 	int first_call = 0;
 
 	/* call does not exist */
@@ -288,6 +634,7 @@ int mncc_recv_internal(struct osmocom_ms *ms, int msg_type, void *arg)
 			return -ENOMEM;
 		call->ms = ms;
 		call->callref = data->callref;
+		call->type = GSM_CALL_T_UNKNOWN;
 		llist_add_tail(&call->entry, &call_list);
 	}
 
@@ -395,51 +742,6 @@ int mncc_recv_internal(struct osmocom_ms *ms, int msg_type, void *arg)
 			cause = GSM48_CC_CAUSE_USER_BUSY;
 			goto release;
 		}
-		/* select first supported speech_ver */
-		if ((data->fields & MNCC_F_BEARER_CAP)) {
-			int i;
-
-			for (i = 0; data->bearer_cap.speech_ver[i] >= 0; i++) {
-
-				temp = mncc_get_bearer(set,
-					data->bearer_cap.speech_ver[i]);
-				if (temp < 0)
-					continue;
-				if (temp == 5 || temp == 1) { /* half */
-					/* only the first half rate */
-					if (speech_ver_half < 0)
-						speech_ver_half = temp;
-				} else {
-					/* only the first full rate */
-					if (speech_ver < 0)
-						speech_ver = temp;
-				}
-			}
-			/* half and full given */
-			if (speech_ver_half >= 0 && speech_ver >= 0) {
-				if (set->half_prefer) {
-					LOGP(DMNCC, LOGL_INFO, " both supported"
-						" codec rates are given, using "
-						"preferred half rate\n");
-					speech_ver = speech_ver_half;
-				} else
-					LOGP(DMNCC, LOGL_INFO, " both supported"
-						" codec rates are given, using "
-						"preferred full rate\n");
-			} else if (speech_ver_half < 0 && speech_ver < 0) {
-				LOGP(DMNCC, LOGL_INFO, " no supported codec "
-					"rate is given\n");
-			/* only half rate is given, use it */
-			} else if (speech_ver_half >= 0) {
-				LOGP(DMNCC, LOGL_INFO, " only supported half "
-					"rate codec is given, using it\n");
-				speech_ver = speech_ver_half;
-			/* only full rate is given, use it */
-			} else {
-				LOGP(DMNCC, LOGL_INFO, " only supported full "
-					"rate codec is given, using it\n");
-			}
-		}
 		/* presentation allowed if present == 0 */
 		if (data->calling.present || !data->calling.number[0])
 			l23_vty_ms_notify(ms, "Incoming call (anonymous)\n");
@@ -456,16 +758,26 @@ int mncc_recv_internal(struct osmocom_ms *ms, int msg_type, void *arg)
 			data->calling.number, call->callref);
 		memset(&mncc, 0, sizeof(struct gsm_mncc));
 		mncc.callref = call->callref;
-		/* only include bearer cap, if not given in setup
-		 * or if multiple codecs are given
-		 * or if not only full rate
-		 * or if given codec is unimplemented
-		 */
-		if (!(data->fields & MNCC_F_BEARER_CAP) || speech_ver < 0)
-			mncc_set_bearer(ms, -1, &mncc);
-		else if (data->bearer_cap.speech_ver[1] >= 0
-		      || speech_ver != 0)
-			mncc_set_bearer(ms, speech_ver, &mncc);
+		/* Bearer capability (optional) */
+		if (mncc_handle_bcap(&mncc, data, &ms->settings) != 0) {
+			cause = GSM48_CC_CAUSE_INCOMPAT_DEST;
+			goto release;
+		}
+		switch (mncc.bearer_cap.transfer) {
+		case GSM48_BCAP_ITCAP_SPEECH:
+			call->type = GSM_CALL_T_VOICE;
+			break;
+		case GSM48_BCAP_ITCAP_UNR_DIG_INF:
+		case GSM48_BCAP_ITCAP_3k1_AUDIO:
+			call->type = GSM_CALL_T_DATA;
+			break;
+		case GSM48_BCAP_ITCAP_FAX_G3:
+			call->type = GSM_CALL_T_DATA_FAX;
+			break;
+		default:
+			call->type = GSM_CALL_T_UNKNOWN;
+			break;
+		}
 		/* CC capabilities (optional) */
 		if (ms->settings.cc_dtmf) {
 			mncc.fields |= MNCC_F_CCCAP;
@@ -532,7 +844,8 @@ int mncc_recv_internal(struct osmocom_ms *ms, int msg_type, void *arg)
 	return 0;
 }
 
-int mncc_call(struct osmocom_ms *ms, char *number)
+int mncc_call(struct osmocom_ms *ms, const char *number,
+	      enum gsm_call_type call_type)
 {
 	struct gsm_call *call;
 	struct gsm_mncc setup;
@@ -552,6 +865,7 @@ int mncc_call(struct osmocom_ms *ms, char *number)
 		return -ENOMEM;
 	call->ms = ms;
 	call->callref = new_callref++;
+	call->type = call_type;
 	call->init = true;
 	llist_add_tail(&call->entry, &call_list);
 
@@ -563,7 +877,8 @@ int mncc_call(struct osmocom_ms *ms, char *number)
 		/* emergency */
 		setup.emergency = 1;
 	} else {
-		LOGP(DMNCC, LOGL_INFO, "Make call to %s\n", number);
+		LOGP(DMNCC, LOGL_INFO, "Make %s call to %s\n",
+		     gsm_call_type_str[call_type], number);
 		/* called number */
 		setup.fields |= MNCC_F_CALLED;
 		if (number[0] == '+') {
@@ -576,7 +891,10 @@ int mncc_call(struct osmocom_ms *ms, char *number)
 		OSMO_STRLCPY_ARRAY(setup.called.number, number);
 
 		/* bearer capability (mandatory) */
-		mncc_set_bearer(ms, -1, &setup);
+		if (call_type == GSM_CALL_T_VOICE)
+			mncc_set_bcap_speech(&setup, &ms->settings, -1);
+		else
+			mncc_set_bcap_data(&setup, &ms->settings, call_type);
 
 		/* CLIR */
 		if (ms->settings.clir)
@@ -727,7 +1045,8 @@ int mncc_retrieve(struct osmocom_ms *ms, int number)
  * DTMF
  */
 
-static int dtmf_statemachine(struct gsm_call *call, struct gsm_mncc *mncc)
+static int dtmf_statemachine(struct gsm_call *call,
+			     const struct gsm_mncc *mncc)
 {
 	struct osmocom_ms *ms = call->ms;
 	struct gsm_mncc dtmf;
