@@ -4,6 +4,7 @@
 
 struct osmo_fsm_inst;
 struct l1sched_state;
+struct l1gprs_state;
 struct msgb;
 
 struct trxcon_inst {
@@ -18,6 +19,9 @@ struct trxcon_inst {
 
 	/* The L1 scheduler */
 	struct l1sched_state *sched;
+	/* GPRS state (MAC layer) */
+	struct l1gprs_state *gprs;
+
 	/* PHY interface (e.g. TRXC/TRXD) */
 	void *phyif;
 	/* L2 interface (e.g. L1CTL) */
@@ -30,6 +34,7 @@ struct trxcon_inst {
 	struct {
 		uint16_t band_arfcn;
 		uint8_t tx_power;
+		uint8_t tsc; /* only valid for DCCH/PDCH */
 		int8_t ta;
 	} l1p;
 
@@ -46,11 +51,12 @@ enum trxcon_log_cat {
 	TRXCON_LOGC_L1D,	/* L1CTL data */
 	TRXCON_LOGC_SCHC,	/* l1sched control */
 	TRXCON_LOGC_SCHD,	/* l1sched data */
+	TRXCON_LOGC_GPRS,	/* l1gprs logging */
 };
 
 void trxcon_set_log_cfg(const int *logc, unsigned int logc_num);
 
-struct trxcon_inst *trxcon_inst_alloc(void *ctx, unsigned int id, uint32_t fn_advance);
+struct trxcon_inst *trxcon_inst_alloc(void *ctx, unsigned int id);
 void trxcon_inst_free(struct trxcon_inst *trxcon);
 
 int trxcon_l1ctl_receive(struct trxcon_inst *trxcon, struct msgb *msg);
