@@ -1,7 +1,7 @@
 /*
  * l1gprs - GPRS layer 1 implementation
  *
- * (C) 2022-2023 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
+ * (C) 2022-2024 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * Author: Vadim Yanitskiy <vyanitskiy@sysmocom.de>
  *
  * All Rights Reserved
@@ -344,19 +344,6 @@ static void l1gprs_remove_tbf_pending_req(struct l1gprs_state *gprs, struct l1gp
 		  LOG_TBF_ARGS(preq), preq->start_fn);
 }
 
-/* Check if the current TDMA Fn is past the start TDMA Fn.
- * Based on fn_cmp() implementation from osmo-pcu.git, simplified. */
-static bool l1gprs_check_fn(uint32_t current, uint32_t start)
-{
-	const uint32_t thresh = GSM_TDMA_HYPERFRAME / 2;
-
-	if ((current < start && (start - current) < thresh) ||
-	    (current > start && (current - start) > thresh))
-		return false;
-
-	return true;
-}
-
 /* Check the list of pending TBFs and move those with expired Fn to the active list */
 static void l1gprs_check_pending_tbfs(struct l1gprs_state *gprs, uint32_t fn)
 {
@@ -364,7 +351,7 @@ static void l1gprs_check_pending_tbfs(struct l1gprs_state *gprs, uint32_t fn)
 	struct l1gprs_tbf *tbf;
 
 	llist_for_each_entry_safe(preq, tmp, &gprs->tbf_list_pending, list) {
-		if (!l1gprs_check_fn(fn, preq->start_fn))
+		if (gsm0502_fncmp(fn, preq->start_fn) < 0)
 			continue;
 
 		LOGP_GPRS(gprs, LOGL_INFO,
@@ -669,6 +656,45 @@ int l1gprs_handle_ul_block_req(struct l1gprs_state *gprs,
 	return 0;
 }
 
+struct msgb *l1gprs_handle_ul_block_cnf(struct l1gprs_state *gprs,
+					uint32_t fn, uint8_t tn,
+					const uint8_t *data,
+					size_t data_len)
+{
+	const struct l1gprs_pdch *pdch = NULL;
+	struct l1ctl_gprs_ul_block_cnf *l1bc;
+	struct msgb *msg;
+
+	OSMO_ASSERT(tn < ARRAY_SIZE(gprs->pdch));
+	pdch = &gprs->pdch[tn];
+
+	LOGP_PDCH(pdch, LOGL_DEBUG, "Rx UL BLOCK.cnf (fn=%u)\n", fn);
+
+	if (pdch->ul_tbf_count + pdch->dl_tbf_count == 0) {
+		LOGP_PDCH(pdch, LOGL_ERROR,
+			  "Rx UL BLOCK.cnf (fn=%u), but this PDCH has no active TBFs\n",
+			  fn);
+		return NULL;
+	}
+
+	msg = l1gprs_l1ctl_msgb_alloc(L1CTL_GPRS_UL_BLOCK_CNF);
+	if (OSMO_UNLIKELY(msg == NULL)) {
+		LOGP_GPRS(gprs, LOGL_ERROR, "l1gprs_l1ctl_msgb_alloc() failed\n");
+		return NULL;
+	}
+
+	l1bc = (void *)msgb_put(msg, sizeof(*l1bc));
+	*l1bc = (struct l1ctl_gprs_ul_block_cnf) {
+		.fn = htonl(fn),
+		.tn = tn,
+	};
+
+	if (data != NULL && data_len > 0)
+		memcpy(msgb_put(msg, data_len), data, data_len);
+
+	return msg;
+}
+
 /* Check if a Downlink block is a PTCCH/D (see 3GPP TS 45.002, table 6) */
 #define BLOCK_IND_IS_PTCCH(ind) \
 	(((ind)->hdr.fn % 104) == 12)
@@ -767,11 +793,11 @@ struct msgb *l1gprs_handle_rts_ind(struct l1gprs_state *gprs, uint32_t fn, uint8
 	struct l1ctl_gprs_rts_ind *l1bi;
 	struct msgb *msg;
 
+	OSMO_ASSERT(tn < ARRAY_SIZE(gprs->pdch));
 	pdch = &gprs->pdch[tn];
 
 	LOGP_PDCH(pdch, LOGL_DEBUG,
-		  "Rx RTS.ind (%s, fn=%u, usf=%u)\n",
-		  ((fn % 104) == 12) ? "PTCCH" : "PDTCH",
+		  "Rx RTS.ind (PDTCH, fn=%u, usf=%u)\n",
 		  fn, usf);
 
 	l1gprs_check_pending_tbfs(gprs, fn);
